﻿using System;
using System.Collections.Generic;
using System.DirectoryServices.AccountManagement;
using System.Linq;
using System.Web.Security;

namespace WinntSecurityProviders
{
    public class WindowsRoleProvider : RoleProvider
    {
        private SecurityProviderHelpers.AuthenticationScope scope;
        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            scope = SecurityProviderHelpers.ToAuthenticationScope(config["scope"]);
            base.Initialize(name, config);
        }
        
        private UserPrincipal AsUserPrincipal(string username, out PrincipalContext context)
        {
            SecurityProviderHelpers.DomainUser dn = new SecurityProviderHelpers.DomainUser(username);

            if (scope == SecurityProviderHelpers.AuthenticationScope.Domain)
            {
                string domainName = SecurityProviderHelpers.FriendlyDomainToLdapDomain(dn.DomainMame);
                context = new PrincipalContext(ContextType.Domain, domainName);
            }
            else
            {
                context = new PrincipalContext(ContextType.Machine, dn.DomainMame);
            }

            return UserPrincipal.FindByIdentity(context, IdentityType.SamAccountName, dn.PrincipalName);
        }
        
        public override bool IsUserInRole(string username, string roleName)
        {
            try
            {
                PrincipalContext ctx;
                UserPrincipal user = AsUserPrincipal(username, out ctx);

                var KnownRoles = RoleConfigurationHelper.GetRoleGroups();

                if (!KnownRoles.Any(x => string.Equals(x.RoleName, roleName, StringComparison.OrdinalIgnoreCase)))
                {
                    throw new ArgumentException(String.Format("Role '{0}' is not mapped to any windows group", roleName), "RoleName");
                }

                var role = KnownRoles.First(x => string.Equals(x.RoleName, roleName, StringComparison.OrdinalIgnoreCase));

                GroupPrincipal group = GroupPrincipal.FindByIdentity(ctx, role.IdentityType, role.WindowsGroupName);

                return group.GetMembers(true).Any(p => p.Sid == user.Sid);
            }
            catch
            {
                return false;
            }
        }

        public override string[] GetRolesForUser(string username)
        {
            try
            {
                PrincipalContext ctx;
                UserPrincipal user = AsUserPrincipal(username, out ctx);
                List<string> result = new List<string>();

                foreach (var role in RoleConfigurationHelper.GetRoleGroups())
                {
                    GroupPrincipal group = GroupPrincipal.FindByIdentity(ctx, role.IdentityType, role.WindowsGroupName);
                    if (group.GetMembers(true).Any(p => p.Sid == user.Sid))
                    {
                        result.Add(role.RoleName);
                    }
                    group.Dispose();
                }

                user.Dispose();

                return result.ToArray();
            }
            catch
            {
                return new string[0];
            }
        }

        public override string[] GetAllRoles()
        {
            return (from role in RoleConfigurationHelper.GetRoleGroups() select role.RoleName).ToArray();
        }

        public override string[] GetUsersInRole(string roleName)
        {
            try
            {
                var roleGroup = RoleConfigurationHelper.GetRoleGroups().First(x => string.Equals(x.RoleName, roleName, StringComparison.OrdinalIgnoreCase)).WindowsGroupName;

                SecurityProviderHelpers.DomainUser dn = new SecurityProviderHelpers.DomainUser(roleGroup);
                PrincipalContext ctx;

                if (scope == SecurityProviderHelpers.AuthenticationScope.Domain)
                {
                    string domainName = SecurityProviderHelpers.FriendlyDomainToLdapDomain(dn.DomainMame);
                    ctx = new PrincipalContext(ContextType.Domain, domainName);
                }
                else
                {
                    ctx = new PrincipalContext(ContextType.Machine, dn.DomainMame);
                }
                GroupPrincipal group = new GroupPrincipal(ctx, dn.PrincipalName);

                return group.GetMembers(true).Select(x => x.SamAccountName).ToArray();
            }
            catch
            {
                return new string[0];
            }
        }

        #region Not implemented methods and properties
        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        public override string ApplicationName
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override void CreateRole(string roleName)
        {
            throw new NotImplementedException();
        }

        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            throw new NotImplementedException();
        }

        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            throw new NotImplementedException();
        }

        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        public override bool RoleExists(string roleName)
        {
            throw new NotImplementedException();
        }
        #endregion
    }
}
